#!/bin/sh
#
# This script write an udev rules file to make interfaces names persistant and user-customizable
#
# Copyright (C) 2006 Jean-Philippe Guillemin <jp.guillemin@free.fr> and Mark Colclough <m.s.colclough@bham.ac.uk>
#
# This file is free software; the copyright holder gives unlimited
# permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, to the extent permitted by law; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.
#

# version 1.0 30/06/2006

VERBOSE="no"
NETRULES=/etc/udev/rules.d/60-interfaces.rules
LOCK=/dev/.udev/lock-${NETRULES##*/}
SYSNET=/sys/class/net

# Log a string via the syslog facility.
log()
{
  [ "$VERBOSE" = "yes" ] && /usr/bin/logger -p user.$1 -t "net-mapper[$$]" -- "$2"
}


touch $NETRULES 1>&2 2>/dev/null
if [ -w $NETRULES ]; then


	# Get a lock for the rules file
	for t in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19; do
		mkdir $LOCK 2>/dev/null && break
		log info "waiting for lock $LOCK"
		sleep 1
	done
	log info "got (or stole) lock $LOCK"


	# We write network interfaces udev rules header
	log info "writing rule header"
cat << "EOF" > $NETRULES
# Network interface naming.  Rules are added automatically as interfaces
# are detected.  You may edit this file, with care.

EOF


	# We write network interfaces udev rules from information in sysfs

	for netif in $SYSNET/* ; do
   NAME=${netif##*/}
   MAC=$(cat $netif/address)
			if [[ $MAC != 00:00:00:00:00:00 && $MAC == ??:??:??:??:??:?? ]]; then
					echo "ACTION==\"add\", SUBSYSTEM==\"net\", DRIVERS==\"?*\", ATTRS{address}==\"$MAC\", NAME=\"$NAME\", GOTO=\"known_interfaces_end\"" >> $NETRULES
					log info "writing rule for interface $sysfsname"
			fi
	done

	# Now the generic rule
	echo "" >> $NETRULES
	echo "ACTION==\"add\", KERNEL==\"eth*\", PROGRAM=\"/lib/udev/net-interface-finder.sh \$env{INTERFACE}\", NAME=\"\$result\"" >> $NETRULES
	echo "ACTION==\"add\", KERNEL==\"eth*\", RUN+=\"/lib/udev/net-mapper.sh\"" >> $NETRULES

	# Finally the footer
	echo "" >> $NETRULES
	echo "LABEL=\"known_interfaces_end\"" >> $NETRULES


	rmdir $LOCK
	log info "removed lock $LOCK"

fi

exit 0


# ACTION=="add", KERNEL=="eth*", PROGRAM="/bin/bash -c 'for ((a=0; a <= 40 ; a++)) ; do echo eth0eth1eth2eth3eth4eth5eth6eth7eth8eth9eth10eth11 | grep -v eth$${a} >/dev/null && break ; done ; echo eth$${a}'", NAME="$result"
